#ifndef SC_SIGNAL_H
#define SC_SIGNAL_H

namespace sc_core {

class sc_signal_base
{
};

template <class T>
class sc_signal: public sc_signal_base
{
    public:

    // Constructors

    sc_signal();
    explicit sc_signal(const char*);

    // Access methods

    T read() const {
        return current_value;
    }

    void write(T t) {
        next_value = t;
    }

    // syntaxic sugar

    operator T () const
    { return read(); }

    sc_signal<T>& operator = ( const T& a )
    { write( a ); return *this; }

    sc_signal<T>& operator = ( const sc_signal<T>& a )
    { write( a.read() ); return *this; }

    // Event checking

    bool event() const {
        previous_value != current_value;
    }

    private:

    void update() {
        previous_value = current_value;
        current_value = next_value;
    }

    T previous_value;
    T current_value;
    T next_value;

    // Disabled

    sc_signal(const sc_signal<T>&);
};

template <>
class sc_signal<bool>: public sc_signal_base
{
    public:

    // Constructors

    sc_signal();
    explicit sc_signal(const char*);

    // Access methods

    bool read() const {
        return current_value;
    }

    void write(bool v) {
        next_value = v;
    }

    // syntaxic sugar

    operator bool () const
    { return read(); }

    sc_signal<bool>& operator = ( bool a )
    { write( a ); return *this; }

    sc_signal<bool>& operator = ( const sc_signal<bool>& a )
    { write( a.read() ); return *this; }

    // Event checking

    bool event() const {
        return previous_value != current_value;
    }

    bool posedge() const {
        return current_value == true && current_value != previous_value;
    }

    bool negedge() const {
        return current_value == false && current_value != previous_value;
    }

    protected:

    bool previous_value;
    bool current_value;
    bool next_value;

    private:

     void update() {
        previous_value = current_value;
        current_value = next_value;
    }

    // Disabled

    sc_signal(const sc_signal<bool>&);
};

template <>
class sc_signal<sc_logic>: public sc_signal_base
{
    public:

    // Constructors

    sc_signal();
    explicit sc_signal(const char*);

    // Access methods

    sc_logic read() const {
        return current_value;
    }

    void write(sc_logic v) {
        next_value = v;
    }

    // syntaxic sugar

    operator sc_logic () const
    { return read(); }

    sc_signal<sc_logic>& operator = ( sc_logic a )
    { write( a ); return *this; }

    sc_signal<sc_logic>& operator = ( const sc_signal<sc_logic>& a )
    { write( a.read() ); return *this; }

    // Event checking

    bool event()const {
        return previous_value != current_value;
    }

    bool posedge() const {
        return current_value == SC_LOGIC_1 && current_value != previous_value;
    }
    bool negedge() const {
        return current_value == SC_LOGIC_0 && current_value != previous_value;
    }

    private:

    void update() {
        previous_value = current_value;
        current_value = next_value;
    }

    T previous_value;
    T current_value;
    T next_value;

    // Disabled

    sc_signal(const sc_signal<sc_logic>&);
};


typedef sc_signal<sc_logic> sc_signal_resolved;

template <int W>
class sc_signal_rv: public sc_signal<sc_lv<W> >
{
    public:
    sc_signal_rv(){}

    private:

    // Disabled
    sc_signal_rv(const sc_signal_rv&);
};

}

#endif
